sync

package
v0.1.1 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Feb 7, 2024 License: MIT Imports: 1 Imported by: 0

Documentation

Overview

The package sync provides same features as the standard sync package, but with generics.

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func OnceFunc

func OnceFunc(f func()) func()

OnceFunc returns a function that invokes f only once. The returned function may be called concurrently.

If f panics, the returned function will panic with the same value on every call.

func OnceValue

func OnceValue[T any](f func() T) func() T

OnceValue returns a function that invokes f only once and returns the value returned by f. The returned function may be called concurrently.

If f panics, the returned function will panic with the same value on every call.

func OnceValues

func OnceValues[T1, T2 any](f func() (T1, T2)) func() (T1, T2)

OnceValues returns a function that invokes f only once and returns the values returned by f. The returned function may be called concurrently.

If f panics, the returned function will panic with the same value on every call.

Types

type Cond

type Cond = sync.Cond

Cond is an alias of sync.Cond.

type Locker

type Locker = sync.Locker

Locker is an alias of sync.Locker.

type Map

type Map[K comparable, V any] struct {
	// contains filtered or unexported fields
}

Map is a generics-enabled version of sync.Map.

func (*Map[K, V]) Delete

func (m *Map[K, V]) Delete(key K)

Delete deletes the value for a key.

func (*Map[K, V]) Load

func (m *Map[K, V]) Load(key K) (value V, ok bool)

Load returns the value stored in the map for a key, or nil if no value is present. The ok result indicates whether value was found in the map.

func (*Map[K, V]) LoadAndDelete

func (m *Map[K, V]) LoadAndDelete(key K) (value V, loaded bool)

LoadAndDelete deletes the value for a key, returning the previous value if any. The loaded result reports whether the key was present.

func (*Map[K, V]) LoadOrStore

func (m *Map[K, V]) LoadOrStore(key K, value V) (actual V, loaded bool)

LoadOrStore returns the existing value for the key if present. Otherwise, it stores and returns the given value. The loaded result is true if the value was loaded, false if stored.

func (*Map[K, V]) Range

func (m *Map[K, V]) Range(f func(key K, value V) bool)

Range calls f sequentially for each key and value present in the map. If f returns false, range stops the iteration.

Range does not necessarily correspond to any consistent snapshot of the Map's contents: no key will be visited more than once, but if the value for any key is stored or deleted concurrently (including by f), Range may reflect any mapping for that key from any point during the Range call. Range does not block other methods on the receiver; even f itself may call any method on m.

Range may be O(N) with the number of elements in the map even if f returns false after a constant number of calls.

func (*Map[K, V]) Store

func (m *Map[K, V]) Store(key K, value V)

Store sets the value for a key.

type Mutex

type Mutex = sync.Mutex

Mutex is an alias of sync.Mutex.

type Once

type Once = sync.Once

Once is an alias of sync.Once.

type Pool

type Pool[T any] struct {
	// New optionally specifies a function to generate
	// a value when Get would otherwise return zero value.
	// It may not be changed concurrently with calls to Get.
	New func() T
	// contains filtered or unexported fields
}

A Pool is a generics-enabled version of sync.Pool.

func (*Pool[T]) Get

func (p *Pool[T]) Get() T

Get selects an arbitrary item from the Pool, removes it from the Pool, and returns it to the caller. Get may choose to ignore the pool and treat it as empty. Callers should not assume any relation between values passed to Put and the values returned by Get.

If Get would otherwise return zero value and p.New is non-nil, Get returns the result of calling p.New.

func (*Pool[T]) Put

func (p *Pool[T]) Put(x T)

Put adds x to the pool.

type RWMutex

type RWMutex = sync.RWMutex

Pool is an alias of sync.Pool.

type WaitGroup

type WaitGroup = sync.WaitGroup

WaitGroup is an alias of sync.WaitGroup.

Jump to